/****************************************************************************
 *
 * Generic Protocol Engine and Upline Data Formats
 *
 ***************************************************************************/

#include "machdef.h"
#include "types.h"

#include "engine.h"                       /* engine definitions */
#include "engvar.h"                       /* engine variables */

WORD debugFlag;

/****************************************************************************
 *
 * Voids and such
 *
 ***************************************************************************/

void  PassData ();
void  CharFlip ();
WORD  LoadData ();


void  Mode0Proc ();
void  Mode1Proc ();
void  Mode2Proc ();
void  Mode3Proc ();
void  Mode4Proc ();
void  Mode5Proc ();
void  Mode6Proc ();
void  Mode7Proc ();
void  LDEProc ();
void  LDAProc ();
void  SSFProc ();
void  EXTProc ();
void  LDCProc ();
void  FGProc ();
void  BGProc ();
void  PaintProc ();

void  SendEsc ();
void  SendEcho ();
void  SendUsm ();
void  SendTouch ();
void  SendExt ();

void  XYAdjust ();
void  FormFeed ();
void  CReturn ();
void  SetMargin ();
void  Restore ();
void  InitMode ();
void  ModeCom ();
void  ModeCur ();

#ifdef	LEVEL1
void 	Mode8Proc ();
void 	Mode9Proc ();
void	Mode10Proc ();
void	Mode11Proc ();
/* void 	Mode12Proc (); */
void	Mode13Proc ();
void	EscIProc ();
void  EscQProc ();
void  EscVProc ();
void  FgPalProc ();
void  BgPalProc ();
void  PalProc ();
void	CentXProc ();
void	CentYProc ();
void  ClipProc ();
void 	DelayProc ();
void 	SendTermStats ();
void 	SendWord ();
void	SerialDelay ();
#endif

/****************************************************************************
 *
 * void InitPad ( flow, width, sub )
 *
 * Intialize the pad
 *
 *    Entry :
 *
 *             flow        True if flow control is available
 *             width       virtual terminal width
 *             sub         terminal subtype to report
 *
 ***************************************************************************/

void InitPad ( flow, width, sub )

BYTE  flow;
WORD  width, sub;

{
   /* virtual terminal intialization */

	if ( width < 512 ) width = 512;
	
   platoXMax = width;
   platoYMax = 511;                    /* good constant to use */
   ttyMode =  True;                    /* tty mode */
   screenMode = SCREWRITE;             /* mode rewrite */
   currentX = 0;
   currentY = 480;                     /* top of screen - 2 lines */
   screenTouch = False;			/* touch not enabled */
   intMask = 0xd0;			/* interrupt mask */

   textCharset = 0;                    /* m0 */
   textDir = FORWARD;
   textAxis = HORIZONTAL;
   textSize = 0;
   textMargin = 0;

   memAddr = 0;
   subType = sub;
   centerX = ( width - 512 ) / 2;		/* center for 512 x 512 screen */
   centerY = 0;

#ifdef   LEVEL1
   upEnhance = False;                  /* not enhanced yet */
   wordCoord = False;                  /* not in word coordinate mode */
   palHold = False;							/* not preserving the palette */
#endif

   /* engine initialization */

   escPending = False;
   flowActive = False;
   flowAvail = flow;                   /* if allowed or not */
   memConv = False;                    /* no mem conversion yet */
   ModeCom ( MODE3 );                  /* command mode mode 3 */
}

/****************************************************************************
 *
 * void CharProcess ( value )
 *
 * process a character from the port 
 *
 *    Entry :
 *
 *             value       character value being processed
 *
 ***************************************************************************/

void CharProcess ( value )

WORD  value;

{
   BYTE  clearEsc = True;              /* true if we clear escape flag */

   value &= 0x7f;                      /* mask off parity bit */
	
   if ( escPending ) {                 /* escape already been seen */
      switch ( value ) {
         case 0x01 : 
         case 0x03 : 
         				ttyMode = False;
#ifdef	LEVEL1
							upEnhance = False;
							CheckPoly ();
#endif
         				break;
         case 0x02 : ttyMode = True; break;
         case 0x0c : ClearPort ();
#ifdef	LEVEL1
							CheckPoly ();
							ClearStyles ();
#endif
							break;
         case 0x11 : 
#ifdef   LEVEL1
							CheckPoly ();
                     if ( !upEnhance ) 
#endif
                        screenMode = SCINVERSE;
                     break;
         case 0x12 : 
#ifdef   LEVEL1
							CheckPoly ();
                     if ( !upEnhance )
#endif
                        screenMode = SCWRITE; 
                     break;
         case 0x13 : 
#ifdef   LEVEL1
							CheckPoly ();
                     if ( !upEnhance )
#endif
                        screenMode = SCERASE; 
                     break;
         case 0x14 : 
#ifdef   LEVEL1
							CheckPoly ();
                     if ( !upEnhance )
#endif
                        screenMode = SCREWRITE; 
                     break;
         case 0x1b : clearEsc = False; break;
         case 0x32 : ModeCur ( LDC ); break;
#ifdef   LEVEL1
         case 0x34 : wordCoord = True; break;
         case 0x35 : wordCoord = False; break;
         case 0x36 : palHold = False; break;
         case 0x37 : CheckPoly ();
         				ClearStyles (); 
         				break;
         
         case 0x38 : CheckPoly ();
         				ModeCur ( DELAY ); 
         				break;
#endif
         case 0x40 : XYAdjust ( 0,5 ); break;
         case 0x41 : XYAdjust ( 0,-5 ); break;
         case 0x42 :                   /* charset m0 */
         case 0x43 :
         case 0x44 :
         case 0x45 :
         case 0x46 : 
         case 0x47 :
         case 0x48 :
         case 0x49 : textCharset = value - 0x42; break;
         case 0x4a : textAxis = HORIZONTAL; break;
         case 0x4b : textAxis = VERTICAL; break;
         case 0x4c : textDir = FORWARD; break;
         case 0x4d : textDir = REVERSE; break;
         case 0x4e : textSize = 0; break;
         case 0x4f : textSize = 2; break;
         case 0x50 : ModeCom ( MODE2 );
                     memConv = True;
                     break;
         case 0x51 : ModeCur ( SSF ); break;
         case 0x52 : ModeCur ( EXT ); break;
         case 0x53 : ModeCom ( MODE2 );
                     memConv = False;
                     break;
         case 0x54 : ModeCom ( MODE5 ); break;
         case 0x55 : ModeCom ( MODE6 ); break;
         case 0x56 : ModeCom ( MODE7 ); break;
         case 0x57 : ModeCur ( LDA ); break;
         case 0x59 : ModeCur ( LDE ); break;
         case 0x5a : SetMargin (); break;
         case 0x61 : ModeCur ( FGCOLOR ); break;
         case 0x62 : ModeCur ( BGCOLOR ); break;
         case 0x63 : ModeCur ( PAINT ); break;
#ifdef   LEVEL1
         case 0x64 : screenMode = SCINVERSE; break;
         case 0x65 : screenMode = SCWRITE; break;
         case 0x66 : screenMode = SCERASE; break;
         case 0x67 : screenMode = SCREWRITE; break;
         case 0x69 : ModeCur ( ESCI ); break;
         case 0x6c : ModeCur ( FGPAL ); break;
         case 0x6d : ModeCur ( BGPAL ); break;
         case 0x6e : ModeCur ( SETPAL ); break;
         case 0x71 : ModeCur ( ESCQ ); 
         				break;
         case 0x76 : ModeCur ( ESCV ); break;
#endif
      }
      if ( clearEsc ) escPending = False;
   }

   else if ( value < 0x20 ) {
      switch ( value ) {
         case 0x08 : XYAdjust ( -CHARWIDTH, 0 ); break;
         case 0x09 : XYAdjust ( CHARWIDTH, 0 ); break;
         case 0x0a : XYAdjust ( 0, -CHARHEIGHT ); break;
         case 0x0b : XYAdjust ( 0, CHARHEIGHT ); break;
         case 0x0c : FormFeed (); break;
         case 0x0d : CReturn (); break;
         case 0x19 : ModeCom ( MODE4 ); break;
         case 0x1b : escPending = True; break;
         case 0x1c : ModeCom ( MODE0 ); break;
         case 0x1d : ModeCom ( MODE1 ); break;
         case 0x1f : ModeCom ( MODE3 ); break;
      }
   }

   else if ( LoadData ( value ) ) PassData ();
}

/****************************************************************************
 *
 * void Passdata ()
 *
 * LoadData completed construction of the requested data, pass it on 
 *
 ***************************************************************************/

void PassData ()

{
   switch ( curMode ) {
      case MODE0  :  Mode0Proc (); break;
      case MODE1  :  Mode1Proc (); break;
      case MODE2  :  Mode2Proc (); break;
      case MODE3  :  Mode3Proc (); break;
      case MODE4  :  Mode4Proc (); break;
      case MODE5  :  Mode5Proc (); break;
      case MODE6  :  Mode6Proc (); break;
      case MODE7  :  Mode7Proc (); break;

      case LDC    :  LDCProc (); break;
      case SSF    :  SSFProc (); break;
      case EXT    :  EXTProc (); break;
      case LDA    :  LDAProc (); break;
      case LDE    :  LDEProc (); break;
      case FGCOLOR : FGProc (); break;
      case BGCOLOR : BGProc (); break;
      case PAINT  :  PaintProc (); break;
      
#ifdef	LEVEL1
		case ESCI	:	EscIProc (); break;
		case ESCQ	:	EscQProc (); break;
		case ESCV	:	EscVProc (); break;
		case CENTX	:	CentXProc (); break;
		case CENTY	:	CentYProc (); break;
		case CLIP	:	ClipProc (); break;
		case SETPAL	:	PalProc (); break;
		case FGPAL	: 	FgPalProc (); break;
		case BGPAL	:	BgPalProc (); break;
		case DELAY 	:	DelayProc (); break;
		case MODE8	:	Mode8Proc (); break;
		case MODE9	:	Mode9Proc (); break;
		case MODE10	:	Mode10Proc (); break;
		case MODE11	:	Mode11Proc (); break;
/*		case MODE12	:	Mode12Proc (); break; */
		case MODE13	:	Mode13Proc (); break;
		
#endif
   }
}

/****************************************************************************
 *
 * WORD LoadData ( value )
 *
 * Take the passed data and put it on the end of whatever is being built
 *
 ***************************************************************************/

WORD LoadData ( value )

WORD  value;

{
   switch ( curState ) {
      case LDBYTE    :  return LoadByte ( value );
      case LDWORD    :  return LoadWord ( value );
      case LDCOORD   :  return LoadCoord ( value );
      case LDCOLOR   :  return LoadColor ( value );
      case LDSLOT    :  return LoadSlot ( value );
   }
}

/****************************************************************************
 *
 * WORD LoadByte ( value )
 *
 * Construct a byte and return true
 *
 ***************************************************************************/

WORD LoadByte ( value )

WORD  value;

{
   ldByte = (BYTE) value;
   return True;
}

/****************************************************************************
 *
 * WORD LoadWord ( value )
 *
 * Construct a word and return true when done
 *
 ***************************************************************************/

WORD LoadWord ( value )

WORD  value;

{
   switch ( seqWord ) {
      case 0x00 : ldWord = value & 0x3f;
                  seqWord++;
                  return False;
      case 0x01 : ldWord += (value & 0x3f) << 6;
                  seqWord++;
                  return False;
      case 0x02 : ldWord += (value & 0x3f) << 12;
                  seqWord = 0;
                  return True;
   }
}

/****************************************************************************
 *
 * WORD LoadCoord ( value )
 *
 * Construct a coordinate and return true when done
 *
 ***************************************************************************/

WORD LoadCoord ( value )

WORD  value;

{
   WORD  command = ( value >> 5 ) & 0x03;       /* command field */

#ifdef   LEVEL1
   if ( wordCoord ) return LoadWCoord ( value );
#endif
   value &= 0x1f;

   switch ( command ) {
      case 0x01 : if ( seqCoord ) ldX = ( ldX & 0x1f )+( value << 5 );
                  else {
			ldY = ( ldY & 0x1f ) + ( value << 5 );
			seqCoord = True;
		  }
                  return False;
      case 0x02 : ldX = ( ldX & 0x03e0 ) + value;
                  seqCoord = False;
                  return True;
      case 0x03 : ldY = ( ldY & 0x03e0 ) + value;
                  seqCoord = True;
                  return False;
   }
}

#ifdef   LEVEL1

WORD LoadWCoord ( value )

WORD  value;

{
   DOUBLE   temp;

   if (!seqCoord) temp = ldX;
   else temp = ldY;

   switch ( seqWCoord++ ) {
      case 0x00 : temp = value & 0x3f; break;
      case 0x01 : temp += (value & 0x3f) << 6; break;
      case 0x02 : temp += (value & 0x3f) << 12; break;
   }

   if ( !seqCoord ) ldX = temp;
   else ldY = temp;

   if ( seqWCoord == 0x03 ) {
      seqWCoord = 0;
      if ( seqCoord ) {
         seqCoord = False;
         return True;
      }
      else seqCoord = True;
   }
   return False;
}
#endif


/****************************************************************************
 *
 * WORD LoadColor ( value )
 *
 * Construct a color and return true when done
 *
 ***************************************************************************/

LoadColor ( value )

WORD  value;

{
   switch ( seqColor ) {
      case 0x00 : ldColor.blue = value & 0x3f;
                  seqColor++;
                  return False;

      case 0x01 : ldColor.blue += ( value & 0x03 ) << 6;
                  ldColor.green = ( value & 0x3c) >> 2;
                  seqColor++;
                  return False;

      case 0x02 : ldColor.green += ( value & 0x0f ) << 4;
                  ldColor.red = ( value & 0x30 ) >> 4;
                  seqColor++;
                  return False;

      case 0x03 : ldColor.red += (value & 0x3f ) << 2;
                  seqColor = 0;
                  return True;
   }
}

/****************************************************************************
 *
 * WORD LoadSlot ( value )
 *
 * Construct a slot and return true when done
 *
 ***************************************************************************/

WORD LoadSlot ( value )

WORD  value;

{
   if ( !seqSlot ) {
      ldSlot = value & 0x3f;
      seqSlot = 1;
      return False;
   }
   else {
      ldSlot += ( value & 0x3f ) << 6;
      seqSlot = 0;
      return True;
   }
}

/****************************************************************************
 *
 * void LdInit ( ldType )
 *
 * Initialize a data load of a given type
 *
 ***************************************************************************/

void LdInit ( ldType )

WORD  ldType;

{
   curState = ldType;

   switch ( ldType ) {
      case LDWORD    :  seqWord = 0; break;
      case LDCOORD   :  seqCoord = 0; 
#ifdef   LEVEL1
                        seqWCoord = 0;
#endif
                        break;
      case LDCOLOR   :  
#ifdef	LEVEL1
								CheckPoly ();
#endif			
								seqColor = 0; break;
      case LDSLOT    :  seqSlot = 0; break;
   }
}

/****************************************************************************
 *
 * void XYAdjust ( deltaX, deltaY )
 *
 * Adjust the current x and y
 *
 *    Entry :
 *
 *             deltax      change in x direction
 *             deltay      change in y direction
 *
 ***************************************************************************/

void XYAdjust ( deltaX, deltaY )

WORD  deltaX, deltaY;

{
   if ( textSize )   {                          /* size 2 */
      deltaX += deltaX;
      deltaY += deltaY;
   }

   switch ( PLOT ) {
      case 0x00 : currentX += deltaX;
                  currentY += deltaY;
                  break;

      case 0x01 : currentX -= deltaX;
                  currentY += deltaY;
                  break;

      case 0x02 : currentY += deltaX;
                  currentX += deltaY;
                  break;

      case 0x03 : currentY -= deltaX;  
                  currentX += deltaY;
                  break;
   }
}

/****************************************************************************
 *
 * void SetMargin ()
 *
 * Set the text margin to the current location
 *
 ***************************************************************************/

void SetMargin ()

{
   if ( textAxis == HORIZONTAL ) textMargin = currentX;
   else textMargin = currentY;
}

/****************************************************************************
 *
 * void FormFeed ()
 *
 * Set currentX/currentY to top of page
 *
 ***************************************************************************/

void FormFeed ()

{
   WORD sizeX = CHARWIDTH;
   WORD sizeY = CHARHEIGHT;

   if ( textSize ) {
      sizeX += sizeX;
      sizeY += sizeY;
   }

   switch ( PLOT ) {
      case 0x00 : currentX = 0;
                  currentY = 496;
                  break;

      case 0x01 : currentX = platoXMax - sizeX;
                  currentY = 496;
                  break;

      case 0x02 : currentY = 0;
                  currentX = sizeX - 1;
                  break;

      case 0x03 : currentY = platoYMax - sizeX;
                  currentX = sizeY - 1;
                  break;
   }
}

/****************************************************************************
 *
 * void CReturn ()
 *
 * Set currentX/currentY to margin on next line
 *
 ***************************************************************************/

void CReturn ()

{
   WORD  size = CHARHEIGHT;

   if ( textSize ) size += size;

   switch ( PLOT ) {
      case 0x00 : 
      case 0x01 : currentX = textMargin;
                  currentY -= size;
                  break;
      case 0x02 :
      case 0x03 : currentY = textMargin;
                  currentX += size;
                  break;
   }
}

/****************************************************************************
 *
 * void ModeCom ( mode )
 *
 * Set the command mode to *mode*
 *
 ***************************************************************************/

void ModeCom ( mode )

BYTE  mode;

{  
   InitMode ( mode );
   comMode = curMode = mode;
   comState = curState;
}

/****************************************************************************
 *
 * void ModeCur ( mode )
 *
 * Set the current mode to *mode*
 *
 ***************************************************************************/

void ModeCur ( mode )

BYTE  mode;

{  
   InitMode ( mode );
   curMode = mode;
}

/****************************************************************************
 *
 * void InitMode ( mode )
 *
 * Initialize the current mode params
 *
 ***************************************************************************/

void InitMode ( mode )

BYTE  mode;

{  
#ifdef	LEVEL1
	CheckPoly ();
#endif
   switch ( (WORD) mode ) {
      case MODE0  :  LdInit ( LDCOORD ); break;
      case MODE1  :  LdInit ( LDCOORD ); 
                     seqMode1 = 0;
                     break;
      case MODE2  :  LdInit ( LDWORD );
      				   buildCount = 0;
      				   break;
      case MODE3  :  LdInit ( LDBYTE ); break;
      case MODE4  :  LdInit ( LDCOORD );
                     seqMode4 = 0;
                     break;
      case MODE5  :
      case MODE6  :
      case MODE7  :  LdInit ( LDWORD ); break;
                    

      case LDC    :  LdInit ( LDCOORD ); break;
      case SSF    :  
      case EXT    :  
      case LDA    :  
      case LDE    :  LdInit ( LDWORD ); break;
      case FGCOLOR : 
      case BGCOLOR : LdInit ( LDCOLOR ); break;
      case PAINT  :  LdInit ( LDSLOT ); break;
#ifdef   LEVEL1
		case MODE8	:
		case MODE9  :
		case MODE10 :
		case MODE12 :
		case MODE13 : seqMode = False;
						  LdInit ( LDWORD );
						  break;
		case MODE11 : seqMode = False;
						  LdInit ( LDCOORD );
						  break;
		case ESCI	: seqI = False;
						  LdInit ( LDBYTE );
						  break;
      case ESCQ   : seqQ = False;
      				  LdInit ( LDBYTE ); 
      				  break;
      case ESCV	: seqV = False;
      				  LdInit ( LDBYTE );
      				  break;
      case CENTX  : LdInit ( LDWORD ); break;
      case CENTY  : LdInit ( LDWORD ); break;
      case CLIP   : seqClip = False;
      				  LdInit ( LDCOORD );
      				  break;
      case SETPAL : seqPal = False;
      				  palHold = True;
      case FGPAL 	:
      case BGPAL 	: LdInit ( LDSLOT ); break; 
      case DELAY	: LdInit ( LDWORD ); break;
#endif
   }
}


/****************************************************************************
 *
 * void Restore ()
 *
 * Restore to mode from command
 *
 ***************************************************************************/

void Restore ()

{  
   curMode = comMode;
   curState = comState;
}

/****************************************************************************
 *
 * void Mode0Proc ()
 *
 * Process mode 0 data
 *
 ***************************************************************************/

void Mode0Proc ()

{
   currentX = ldX;
   currentY = ldY;
   PlotDot ( ldX, ldY );
   LdInit ( LDCOORD );
}

/****************************************************************************
 *
 * void Mode1Proc ()
 *
 * Process mode 1 data
 *
 ***************************************************************************/

void Mode1Proc ()

{
   if ( !seqMode1 ) 
      seqMode1 = 1;
   else 
   	PlotLine ( currentX, currentY, ldX, ldY );
   currentX = ldX;
   currentY = ldY;
   LdInit ( LDCOORD );
}

/****************************************************************************
 *
 * void Mode2Proc ()
 *
 * Process mode 2 data
 *
 ***************************************************************************/

void Mode2Proc ()

{
	if ( memConv ) {					/* putting together charset */
		buildChar [ buildCount ++ ] = (WORD) ldWord;
		memAddr += 2;
		if ( buildCount == 8 ) {
			CharFlip ();					/* convert to bitmap */
			LoadAddrChar ( charBitMap );
			buildCount = 0;
		}
	}
   LdInit ( LDWORD );
}

/****************************************************************************
 *
 * void Mode3Proc ()
 *
 * Process mode 3 data
 *
 ***************************************************************************/

void Mode3Proc ()

{
	WORD	tempX;
	
	if ( textDir == REVERSE ) {
		tempX = currentX;
		XYAdjust ( CHARWIDTH, 0 );
		}
   PlotChar ( ldByte );
   if ( textDir == REVERSE ) currentX = tempX;
   XYAdjust ( CHARWIDTH, 0 );
   LdInit ( LDBYTE );
}

/****************************************************************************
 *
 * void Mode4Proc ()
 *
 * Process mode 4 data
 *
 ***************************************************************************/

void Mode4Proc ()

{
   WORD  temp;

   if ( !seqMode4 ) {
      currentX = ldX;
      currentY = ldY;
      seqMode4 = 1;
   }
   else {
      PlotBlock ( currentX, currentY, ldX, ldY );
      seqMode4 = 0;
      temp = textSize;
      textSize = 0;
      XYAdjust ( 0, 15 );
      textSize = temp;
   }
   LdInit ( LDCOORD );
}

/****************************************************************************
 *
 * void Mode5Proc ()
 *
 * Process mode 5 data
 *
 ***************************************************************************/

void Mode5Proc ()

{
   LdInit ( LDWORD );
}

/****************************************************************************
 *
 * void Mode6Proc ()
 *
 * Process mode 6 data
 *
 ***************************************************************************/

void Mode6Proc ()

{
   LdInit ( LDWORD );
}

/****************************************************************************
 *
 * void Mode7Proc ()
 *
 * Process mode 7 data
 *
 ***************************************************************************/

void Mode7Proc ()

{
   LdInit ( LDWORD );
}

/****************************************************************************
 *
 * void LDCProc ()
 *
 * Process LDC command data
 *
 ***************************************************************************/

void LDCProc ()

{
   currentX = ldX;
   currentY = ldY;
   Restore ();
}

/****************************************************************************
 *
 * void SSFProc ()
 *
 * Process SSF command data
 *
 ***************************************************************************/

void SSFProc ()

{
	if ( (ldWord & 0x7c00) == 0x0400 ) {		/* interrupt mask */
		intMask = ( ldWord & 0x00ff );
		screenTouch = ( ldWord & 0x0020 );	/* touch panel */
	}
   Restore ();
}

/****************************************************************************
 *
 * void EXTProc ()
 *
 * Process EXT command data
 *
 ***************************************************************************/

void EXTProc ()

{
   Restore ();
}

/****************************************************************************
 *
 * void LDAProc ()
 *
 * Process LDA command data
 *
 ***************************************************************************/

void LDAProc ()

{
   memAddr = ldWord;
   Restore ();
}

/****************************************************************************
 *
 * void LDEProc ()
 *
 * Process LDE command data
 *
 ***************************************************************************/

void LDEProc ()

{
   switch ( (WORD) ldWord ) {
      case 0x70 : SendEcho ( 12 ); break;
      case 0x71 : SendEcho ( subType ); break;
      case 0x72 : SendEcho ( 0 ); break;
      case 0x73 : SendEcho ( 0x40 ); break;
      case 0x7a : SendUsm ( 0xff ); break;
      case 0x7b : Beep (); break;
      default : SendEcho ( (WORD) ldWord );
   }
   Restore ();
}

/****************************************************************************
 *
 * void FGProc ()
 *
 * Process FGCOLOR command data
 *
 ***************************************************************************/

void FGProc ()

{
	fgColor.red = ldColor.red;
	fgColor.green = ldColor.green;
	fgColor.blue = ldColor.blue;
	FGColor ( fgColor.red, fgColor.green, fgColor.blue );
    Restore ();
}

/****************************************************************************
 *
 * void BGProc ()
 *
 * Process BGCOLOR command data
 *
 ***************************************************************************/

void BGProc ()

{	
	bgColor.red = ldColor.red;
	bgColor.green = ldColor.green;
	bgColor.blue = ldColor.blue;
	BGColor ( bgColor.red, bgColor.green, bgColor.blue );
    Restore ();
}

/****************************************************************************
 *
 * void PaintProc ()
 *
 * Process PAINT command data
 *
 ***************************************************************************/

void PaintProc ()

{
   Restore ();
}

/****************************************************************************
 *
 * SetBit ( byteNum, bitNum )
 *
 * Random routine to set a bit for char conversion
 *
 ***************************************************************************/
 
void SetBit ( byteNum, bitNum )

WORD byteNum, bitNum;

{
	WORD bitVal;
	
	byteNum = 15 - byteNum;
	bitNum = 7 - bitNum;
	bitVal = ( 1 << bitNum );
	charBitMap [ byteNum ] |= (bitVal & 0xff );
}

/**************************************************************************
 *
 *	void CharFlip ()
 *
 *	Flip a character from plato format to a decent bitmap format
 *
 ************************************************************************/

void CharFlip ()

{
	WORD i, wordCount, bitCount, tempWord;
	
	for ( i = 0; i < 16; i ++ ) charBitMap [ i ] = 0;
	
	for ( wordCount = 0; wordCount < 8; wordCount ++ ) {
		tempWord = buildChar [ wordCount ];
		for ( bitCount = 0; bitCount < 16; bitCount ++ ) {
			if ( tempWord & 0x01 ) SetBit ( bitCount, wordCount );
			tempWord = tempWord >> 1;
		}
	}
}


#ifdef   LEVEL1
/****************************************************************************
 ****************************************************************************
 *
 * LEVEL 1 extensions
 *
 ****************************************************************************
 ***************************************************************************/

/****************************************************************************
 *
 * void EscIProc ()
 *
 * Process esc I command data
 *
 ***************************************************************************/

void EscIProc ()

{
	static	WORD	setStyle;
	
	if ( !seqI ) {
		setStyle = (WORD)ldByte;
		if ( setStyle == 0x42 ) LdInit ( LDWORD );
		else LdInit ( LDBYTE );
		seqI = True;
	}
	else {
		ldByte &= 0x3f;
		switch ( setStyle ) {
			case 0x41 : StylePattern ( (WORD)ldByte ); break;
			case 0x42 : StyleThickness ( (WORD)ldWord ); break;
			case 0x43 : StyleCap ( (WORD)ldByte ); break;
			case 0x44 : StyleDash ( (WORD)ldByte ); break;
			case 0x45 : StyleJoin ( (WORD)ldByte ); break;
			case 0x46 : StyleFill ( (WORD)ldByte ); break;
		}
		Restore ();
		seqI = False;
	}
}

/****************************************************************************
 *
 * void EscQProc ()
 *
 * Process esc q command data
 *
 ***************************************************************************/

void EscQProc ()

{

   Restore ();
   switch ( (WORD) ldByte ) {
   	case 0x41 : ModeCom ( MODE8 ); break;
   	case 0x42 : ModeCom ( MODE13 ); break;
   	case 0x43 : ModeCom ( MODE9 ); break;
   	case 0x44 : ModeCom ( MODE10 ); break;
   	case 0x4b : ModeCom ( MODE11 ); break;
   	case 0x76 : SendTermStats (); break;
   	case 0x77 : upEnhance = True; break;
     	case 0x78 : ModeCur ( CENTX ); break;
     	case 0x79 : ModeCur ( CENTY ); break;
     	case 0x7a : ModeCur ( CLIP ); break;
   }
}

/****************************************************************************
 *
 * void EscVProc ()
 *
 * Process esc v command data
 *
 ***************************************************************************/

void EscVProc ()

{
	Restore ();
}

/****************************************************************************
 *
 * void CentXProc ()
 *
 * Process center X command data
 *
 ***************************************************************************/

void CentXProc ()

{
   centerX = (WORD)ldWord;
   Restore ();
}

/****************************************************************************
 *
 * void CentYProc ()
 *
 * Process center Y command data
 *
 ***************************************************************************/

void CentYProc ()

{
   centerY = (WORD)ldWord;
   Restore ();
}

/****************************************************************************
 *
 * void ClipProc ()
 *
 * Process clipping data
 *
 ***************************************************************************/

void ClipProc ()

{
	static WORD	clipX, clipY;
	
	if ( !seqClip ) {
		clipX = ldX;
		clipY = ldY;
		seqClip = True;
		LdInit ( LDCOORD );
	}
	else {
		ClipSet ( clipX, clipY, ldX, ldY );
		seqClip = False;
		Restore ();
	}
}

/****************************************************************************
 *
 * void PalProc ()
 *
 * Set Palette Entry
 *
 ***************************************************************************/

void PalProc ()

{
	static WORD	tempSlot;
	
	if ( !seqPal ) {
		tempSlot = ldSlot;
		seqPal = True;
		LdInit ( LDCOLOR );
	}
	else {
#ifdef	PALETTE
		SetPalSlot ( ldSlot, ldColor . red, ldColor . green, ldColor . blue );
#endif
		Restore ();
	}
}

/****************************************************************************
 *
 * void FgPalProc ()
 *
 * Set foreground to palette slot
 *
 ***************************************************************************/

void FgPalProc ()

{
#ifdef	PALETTE
	SetFGPal ( ldSlot );
#endif
   Restore ();
}

/****************************************************************************
 *
 * void BgPalProc ()
 *
 * Set background to palette slot
 *
 ***************************************************************************/

void BgPalProc ()

{
#ifdef	PALETTE
	SetBGPal ( ldSlot );
#endif
   Restore ();
}

/****************************************************************************
 *
 * void DelayProc ()
 *
 * Delay for *ldWord* msecs
 *
 ***************************************************************************/

void DelayProc ()

{
	DelayPad ( (WORD) ldWord );
   Restore ();
}

/****************************************************************************
 *
 * void Mode8Proc ()
 *
 * Process elliptical arcs
 *
 ***************************************************************************/

void Mode8Proc ()

{
	static WORD	xRadius, yRadius, startAngle;
	
	switch ( seqMode ) {
		case 0x00 :	xRadius = (WORD) ldWord;
						seqMode = 1;
						break;
		case 0x01 : yRadius = (WORD) ldWord;
						seqMode = 2;
						break;
		case 0x02 : startAngle = (WORD) ldWord;
						seqMode = 3;
						break;
		case 0x03 : PlotArc ( xRadius, yRadius, startAngle, (WORD)ldWord );
						seqMode = 0;
						break;
	}
	LdInit ( LDWORD );
}

/****************************************************************************
 *
 * void Mode9Proc ()
 *
 * Process circles
 *
 ***************************************************************************/

void Mode9Proc ()

{
	PlotEllipse ( (WORD) ldWord, (WORD) ldWord );
	LdInit ( LDWORD );
}

/****************************************************************************
 *
 * void Mode10Proc ()
 *
 * Process ellipses
 *
 ***************************************************************************/

void Mode10Proc ()

{
	static WORD	xRadius;
	
	if ( !seqMode ) {
		xRadius = (WORD) ldWord;
		seqMode = True;
	}
	else {
		PlotEllipse ( xRadius, (WORD) ldWord );
		seqMode = False;
		}
	LdInit ( LDWORD );
}

/****************************************************************************
 *
 * void Mode11Proc ()
 *
 * Process boxes
 *
 ***************************************************************************/

void Mode11Proc ()

{
	static WORD	x1,y1,x2,y2;
	
	switch ( seqMode ) {
		case 0x00 :	x1 = ldX;
						y1 = ldY;
						seqMode = 1;
						LdInit ( LDCOORD );
						break;
		case 0x01 : x2 = ldX;
						y2 = ldY;
						seqMode = 2;
						LdInit ( LDWORD );
						break;
		case 0x02 : PlotBox ( x1,y1,x2,y2, (WORD)ldWord );
						seqMode = 0;
						LdInit ( LDCOORD );
						break;
	}
}

/****************************************************************************
 *
 * void Mode13Proc ()
 *
 * Process circular arcs
 *
 ***************************************************************************/

void Mode13Proc ()

{
	static WORD	radius, startAngle;
	
	switch ( seqMode ) {
		case 0x00 :	radius = (WORD) ldWord;
						seqMode = 1;
						break;
		case 0x01 : startAngle = (WORD) ldWord;
						seqMode = 2;
						break;
		case 0x02 : PlotArc ( radius, radius, startAngle, (WORD)ldWord );
						seqMode = 0;
						break;
	}
	LdInit ( LDWORD );
}

#endif

/****************************************************************************
 ****************************************************************************
 *
 * Upline data formats
 *
 ****************************************************************************
 ***************************************************************************/

/****************************************************************************
 *
 * void SendEcho ( value )
 *
 * send *value* back to plato with echo bits set 
 *
 ***************************************************************************/

void SendEcho ( value )

WORD  value;

{
   SendEsc ( (value & 0x7f) + 0x0080 );
}

/****************************************************************************
 *
 * void SendTouch ( touchX, touchY )
 *
 * send course grid touch back to plato 
 *
 ***************************************************************************/

void SendTouch ( touchX, touchY )

WORD  touchX, touchY ;

{
   SendEsc ( (touchY & 0x0f) + ((touchX & 0x0f) << 4) + 0x0100 );
}

/****************************************************************************
 *
 * void SendExt ( value )
 *
 * send *value* back to plato with ext bits set 
 *
 ***************************************************************************/

void SendExt ( value )

WORD  value;

{
   SendEsc ( (value & 0xff) + 0x0200 );
}

/****************************************************************************
 *
 * void SendUsm ( value )
 *
 * send *value* back to plato with usm bits set 
 *
 ***************************************************************************/

void SendUsm ( value )

WORD  value;

{
   SendEsc ( (value & 0x7f) + 0x0380 );
}

/****************************************************************************
 *
 * void SendEsc ( value )
 *
 * send *value* back to plato packaged as an escape sequence
 *
 ***************************************************************************/

void SendEsc ( value )

WORD  value;

{
   SendChar ( 0x1b );
   SendChar ( (value & 0x3f ) + 0x40 );
   SendChar ( ((value & 0x3c0) >> 6 ) + 0x60 );
}

#ifdef	LEVEL1
/****************************************************************************
 *
 * void SendPixRes ( xLoc, yLoc )
 *
 * (xLoc, yLoc) back as a high-res touch key
 *
 ***************************************************************************/

void SendPixRes ( xLoc, yLoc )

WORD  xLoc, yLoc;

{
   SendChar ( 0x1b );
   SendChar ( 0x1e );
   SendChar ( 0xc1 );
   SendWord ( xLoc );
   SendWord ( yLoc );
}

/****************************************************************************
 *
 * void SendTermStats ( )
 *
 * Send back terminal statistics
 *
 ***************************************************************************/

void SendTermStats ( )

{
	WORD	termStats = 0;
	
#ifdef	KERMIT
	termStats = 0x4000;
#endif

#ifdef	USECOLOR
	termStats += 0x2000;
#endif

#ifdef	PALETTE
	termStats += 0x1000;
#endif
	termStats += PALSLOTS;

   SendChar ( 0x1b );
   SerialDelay ();
   SendChar ( 0x1e );
   SerialDelay ();
   SendChar ( 0xe0 );
   SerialDelay ();
   SendWord ( platoXMax & 0x3ff );
   SendWord ( termStats );
}

/****************************************************************************
 *
 * void SendWord ( wordVal )
 *
 * send wordVal back to plato in word format ( ignore bits 18 and 17 )
 *
 ***************************************************************************/

void SendWord ( wordVal )

WORD  wordVal;

{
	SendChar ( 0x40 | ( wordVal & 0x3f ) );
	SerialDelay ();
	SendChar ( 0x40 | ( ( wordVal >> 6 ) & 0x3f ) );
	SerialDelay ();
	SendChar ( 0x40 | ( ( wordVal >> 12 ) & 0x0f ) );
	SerialDelay ();
}

void	SerialDelay ()
{
	DelayPad ( 6 );
}
#endif





















                     
                     






