/****************************************************************************
 *
 * Command Line Parser 
 *
 ***************************************************************************/

#include 	<stdio.h>
#include 	<errno.h>
#include	<string.h>

#include 	"machdef.h"
#include 	"types.h"
#include 	"engine.h"

/**************************************************************************
 *
 *	Global Variables
 *
 *************************************************************************/

int	setBaud;
int	setType;
int	setBase;
int	setWidth;
int	setBatch;
int	use_inet_sock;
char	setPort [80];
char	setFont [80];
char	unix_sock_name [80];
char	inet_host_name [80];
char	inet_serv_name [80];
char	icon_label [25];

unsigned long	inet_host_addr;
int		inet_host_byname;
int		inet_serv_port;
int		inet_serv_byname;

void usage();
int getint();
/**************************************************************************
 **************************************************************************
 *
 *	Routines
 *
 **************************************************************************
 *************************************************************************/

/**************************************************************************
 *
 *	CmdParse ( argc, argv )
 *
 *	Valid Commands :
 *
 *	-b ####			: set character max batch to ####
 *	-d /dev/ttya		: device to use
 *	-f m0.fnt		: font to use for m0/m1
 *	-h name			: use name as inet host
 *	-n			: do not use console for nova or dev
 *	-s ####			: set baud rate to ####
 *	-S name			: chose system to use
 *	-v name			: use name as inet service
 *	-z ####			: set ztstype to ####
 *
 *************************************************************************/

void CmdParse ( argc, argv )

int	argc;
char	*argv [];

{
	int	i,argi,pos,bspec,dspec,fspec,hspec,Hspec,nspec,sspec,Sspec,
		vspec,Vspec,zspec;
	char	*param,option,sys_name[10];

	/* assume nothing is specified */
	bspec=dspec=fspec=hspec=Hspec=nspec=sspec=Sspec=vspec=Vspec=zspec=0;
	inet_host_byname = True;
	inet_serv_byname = True;

	setBaud = 1200;				/* default 1200 baud */
	setType = 122;				/* default Sun type */
	setBase = 0xe340;			/* default base */
	setWidth = 640;				/* default width */
	setBatch = False;			/* turn off batching */
	strcpy(setPort,"/dev/ttya");		/* default port */
	strcpy(setFont,"/nova/lib/fonts/m0.fnt");/* default font */
	use_inet_sock = False;

	/* Don't count file name as argument */

	argi = 1;
	argc--;

	if (argc < 1) {
		usage();
		exit(0);
	}

	while ( argc > 0 ) {			

		/* check option syntax */

		if (argv[argi][0] != '-') {
			fprintf(stderr,"pad: bad option syntax %s\n",
				argv[argi]);
			exit(1);
		}

		option = argv[argi][1];

		if (option == 'n') {
			use_inet_sock = True;
			nspec++;
			argc--;
			argv++;
		} else if ((option == 'b') || (option == 'd') || (option == 'f')
			|| (option == 'h') || (option == 'H') || (option == 's')
			|| (option == 'S') || (option == 'v') || (option == 'V')
			|| (option == 'z')) {

			if (argv[argi][2] == '\0') {
				pos = 0;
				argc--;
				if (argc < 1) {
					fprintf(stderr,"pad: option %s requires a parameter\n",argv[argi]);
					exit(1);
				}
				argi++;
			} else 
				pos = 2;

			param = &(argv[argi][pos]);
		
			switch (option) {

				case 'b' :
					setBatch = getint(param,option);
					bspec++;
					argc--;
					argv++;
					break;

				case 'd' :
					strcpy(setPort,param);
					dspec++;
					argc--;
					argv++;
					break;

				case 'f' :
					strcpy(setFont,param);
					fspec++;
					argc--;
					argv++;
					break;

				case 'h' :
					strcpy(inet_host_name,param);
					strcpy(icon_label,param);
					use_inet_sock = True;
					hspec++;
					argc--;
					argv++;
					break;

				case 'H' :
					inet_host_addr=inet_addr(param);
					use_inet_sock = True;
					inet_host_byname = False;
					Hspec++;
					argc--;
					argv++;
					break;

				case 's' :
					setBaud = getint(param,option);
					sspec++;
					argc--;
					argv++;
					break;

				case 'S' :
					strcpy(sys_name,param);
					Sspec++;
					argc--;
					argv++;
					break;

				case 'v' :
					strcpy(inet_serv_name,param);
					use_inet_sock = True;
					vspec++;
					argc--;
					argv++;
					break;

				case 'V' :
					inet_serv_port = getint(param,option);
					use_inet_sock = True;
					inet_serv_byname = False;
					Vspec++;
					argc--;
					argv++;
					break;

				case 'z' :
					setType = getint(param,option);
					zspec++;
					argc--;
					argv++;
			}
		} else {
			fprintf(stderr,"pad: bad option -%c\n",option);
			exit(1);
		}
	}

	if (((bspec | dspec | fspec | hspec | Hspec | nspec | sspec | Sspec |
		vspec | Vspec | zspec) > 1) || (Hspec && hspec) || 
		(vspec && Vspec)) {
		fprintf(stderr,"pad: conflicting or duplicate options\n");
		exit(1);
	}

	if (Sspec) {
		if (strcmp(sys_name,"cerl") == 0) {
			use_inet_sock = True;
			strcpy(inet_serv_name,"niuc");
			strcpy(inet_host_name,"uinova");
			strcpy(icon_label,"cerl");
		} else if (strcmp(sys_name,"nova") == 0) {
			strcpy(unix_sock_name, "/nova/tmp/console/nova"); 
			if (!(vspec || Vspec))
				strcpy(inet_serv_name, "niun"); 
			if (!(hspec || Hspec))
				strcpy(inet_host_name, "novacon");
			strcpy(icon_label,"nova");
		} else if (strcmp(sys_name,"dev") == 0) {
			strcpy(unix_sock_name,"/nova/tmp/console/dev");
			if (!(vspec || Vspec))
				strcpy(inet_serv_name,"niux");
			if (!(hspec || Hspec))
				strcpy(inet_host_name,"uinova");
			strcpy(icon_label,"dev");
		} else {
			fprintf(stderr,"pad: system name %s unrecognized\n",
				sys_name);
			exit(1);
		}
	} else if (!((hspec || Hspec) && (vspec || Vspec))) {
		fprintf(stderr,"pad: system (-S) or host (-h/-H) and service (-v/-V) required\n");
		exit(1);
	}

	if (Hspec && (inet_host_addr == -1)) {
		fprintf(stderr,"pad: bad internet address\n");
		exit(1);
	}

	if (Vspec && (inet_serv_port < 1)) {
		fprintf(stderr,"pad: bad service number\n");
		exit(1);
	}

	if (sspec && (setBaud % 1200 != 0) || (setBaud == 0) || 
		(setBaud > 9600)) {
		fprintf(stderr,"pad: invalid baud rate %d\n",setBaud);
		exit(1);
	}

	if (zspec && (setType == 116)) {
		fprintf(stderr,"pad: enhanced subtype (116) not allowed\n");
		exit(1);
	} else if (zspec) {
		switch ( setType ) {
			case   0 :	/* ist 2 */
			case   1 :	/* ist 3 */
			case   2 :	/* viking */
				setWidth = 512;
				setBase = 0x2340;
				break;

			case   3 :	/* ast */
			case 122 :	/* ast */
				setWidth = 640;
				setBase = 0xe340;
				break;

			case 117 :	/* sunpad */
				setWidth = 1024;
				setBase = 0x3800;
				break;

			default :
				setWidth = 512;
				setBase = 0x3800;
				break;
		}
	}

	if (!use_inet_sock)
		strcat(icon_label,"@con");

/*
	printf("Batch		%d		%s\n",setBatch,(bspec ? "SET" : ""));
	printf("Device		%s		%s\n",setPort,(dspec ? "SET" : ""));
	printf("Font		%s		%s\n",setFont,(fspec ? "SET" : ""));
	printf("Baud		%d		%s\n",setBaud,(sspec ? "SET" : ""));
	printf("Subtype		%d		%s\n",setType,(zspec ? "SET" : ""));
	printf("Host		%s		%s\n",inet_host_name,(hspec ? "SET" : ""));
	printf("Host addr	%x		%s\n",inet_host_addr,(Hspec ? "SET" : ""));
	printf("Service		%s		%s\n",inet_serv_name,(vspec ? "SET" : ""));
	printf("Service	port	%d		%s\n",inet_serv_port,(Vspec ? "SET" : ""));
	printf("Console		%s		%s\n",(use_inet_sock ? "NO" : "YES"),(nspec ? "SET" : ""));
	printf("System		%s		%s\n",sys_name,(Sspec ? "SET" : ""));
	printf("UnixSock	%s\n",unix_sock_name);
	printf("Icon		%s\n",icon_label);
*/

}

int getint(param,option)
char *param,option;
{
	int	result;
	char	*died;

	result = (int) strtol(param,&died,10);

	if ((died < (param + strlen(param))) || (died == param)) {
		fprintf(stderr,"pad: missing or bad integer parameter ");
		fprintf(stderr,"%s for option -%c\n",param,option);
		exit(1);
	}

	return(result);
}

/**************************************************************************
 *
 *	usage ()
 *
 *	Output the proper usage
 *
 *************************************************************************/

void usage () 

{
	fprintf(stderr,"format:	pad [-Ssystem] [-hhostname | -Hhostaddr] [-vservice | -Vportnum] [-n]\n");
	fprintf(stderr,"\t    [-bmaxbatch] [-ddevicename] [-ffontpath] [-sbaudrate] [-zsubtype]\n\n");
	fprintf(stderr,"\t-bmaxbatch	set batch count\n");
	fprintf(stderr,"\t-ddevicename	set device name\n");
	fprintf(stderr,"\t-ffontname	set font file\n");
	fprintf(stderr,"\t-hhostname	specify internet host by name\n");
	fprintf(stderr,"\t-Hhostaddr	specify internet host by address\n");
	fprintf(stderr,"\t-n		run as non-console\n");
	fprintf(stderr,"\t-sbaudrate	set baud rate\n");
	fprintf(stderr,"\t-Ssystem	specify cerl, nova, or dev system\n");
	fprintf(stderr,"\t-vservice	specify internet service by name\n");
	fprintf(stderr,"\t-Vportnum	specify internet service by port\n");
	fprintf(stderr,"\t-zsubtype	set terminal type\n");
}
