/****************************************************************************
 *
 * 	PLATO Keyboard Processing Routines
 *
 ***************************************************************************/

#include 	"machdef.h"
#include 	"types.h"
#include	"pkeyboard.h"

/****************************************************************************
 *
 * 	Externals
 *
 ***************************************************************************/

extern BYTE	flowActive;		/* true if flow controlled */

/***************************************************************************
 *
 *	Globals
 *
 **************************************************************************/

WORD	FuncKeyMap [] = { 0x13, 0x17, 0x04, 0x05, 0x07, 0x14, 0x03, 0x16,
			  0x0a, 0x1c, 0x08, 0x19, 0x7b, 0x7f, 0x0b, 0x09,
			  0x7d, 0x00, 0x0d, 0x1e, 0x1a, 0x18, 0x02, 0x0e,
			  0x0c, 0x0f, 0x12, 0x1d, 0x01, 0x11, 0x20, 0x1f };

WORD	AltNumMap [] = { 0x3c, 0x3e, 0x5b, 0x5d, 0x24, 0x25, 0x5f, 0x7c,
			 0x2a, 0x28 };

void	SendNum ();
void	SendKey ();
void	SendFunc ();

/**************************************************************************
 *
 *	void SendAscii ( value, state, rawMode )
 *
 *	Parse up the ascii codes and send something useful to plato
 *
 *	Note that this assumes a rational keyboard that returns the
 *	standard codes as described in the AT&T manual
 *
 *************************************************************************/

void	SendAscii ( value, state, rawMode )

WORD	value;
WORD	state;
BYTE	rawMode; /* no longer used, was ignore anyway */

{
	int	shift;

	state &= 0x03;
	shift = ( state == SHIFTED );

	if (( value == 0x04 ) && ( state == ALTED ) ) SendKey ( 0x60 );		/* divide sign */
	else if (( value == 0x04 ) && ( state == (ALTED|SHIFTED))) SendKey ( 0x27 );	/* upside down u */
	else if (( value == 0x08 ) && shift ) SendFunc ( ERASE1 );	/* shift erase */
	else if ( value == 0x09 ) {				/* tab */
		if ( shift ) SendFunc ( CR );
		else SendFunc ( TAB );
	}
	else if ((value == 0x0d) && shift ) SendFunc ( NEXT1 );	/* shift next */
	else if ((value == 0x0d) && ( state == ALTED )) SendKey ( 0x26 );	/* times sign */
	else if ((value == 0x0d) && ( state == (ALTED|SHIFTED))) SendKey ( 0x40 );	/* small u thingie */
	else if ((value == 0x15) && ( state == ALTED ) ) SendFunc ( SUPER );
	else if ((value == 0x16) && ( state == ALTED ) ) SendFunc ( SUB );
	else if ((value == 0x15) && ( state == ( ALTED | SHIFTED) ) ) SendFunc ( SUPER1 );
	else if ((value == 0x16) && ( state == ( ALTED | SHIFTED) )) SendFunc ( SUB1 );
	else if ((value == 0x1b) && shift ) SendKey ( 0x5c );	/* shift code */
	else if ( value == 0x1b ) SendKey ( 0x5e );		/* assign arrow */
	else if (( value == 0x1f ) && ( state == ALTED ) ) SendKey ( 0x7e );	/* delta */
	else if ( (value==0x20) && shift ) SendFunc ( BACKSP );	/* shifted space */
	else if ( value == 0x23 ) SendNum ( 3, SHIFTED );	/* '#' */
	else if ( value == 0x26 ) SendNum ( 7, SHIFTED );	/* '&' */
	else if ( value == 0x27 ) SendKey ( 0x7c );		/* ''' */
	else if (( value == 0x3d ) && ( state == ALTED ) ) SendKey ( 0x23 );	/* sigma */
	else if ( value == 0x40 ) SendNum ( 2, SHIFTED );	/* '@' */
	else if ((value == 0x55) && ( state == ( ALTED | SHIFTED) ) ) SendFunc ( SUPER1 );
	else if ((value == 0x56) && ( state == ( ALTED | SHIFTED) )) SendFunc ( SUB1 );
	else if ( value == 0x5c ) {				/* '\' */
		SendKey ( 0x7b );
		SendKey ( 0x2f );
	}
	else if ( value == 0x5e ) SendNum ( 6, SHIFTED );	/* '^' */
	else if ( value == 0x60 ) {				/* '`' */
		SendKey ( 0x7b );
		SendKey ( 0x71 );
	}
	else if ((value == 0x75) && ( state == ALTED ) ) SendFunc ( SUPER );
	else if ((value == 0x76) && ( state == ALTED )) SendFunc ( SUB );
	else if ( value == 0x7b ) {				/* '{' */
		SendKey ( 0x7b );
		SendKey ( 0x5b );
	}
	else if ( value == 0x7c ) {				/* '|' */
		SendKey ( 0x7b );
		SendKey ( 0x49 );
	}
	else if ( value == 0x7d ) {				/* '}' */
		SendKey ( 0x7b );
		SendKey ( 0x5d );
	}
	else if ( value == 0x7e ) {				/* '~' */
		SendKey ( 0x7b );
		SendKey ( 0x3b );
	}
	else if (( value == 0x7f ) && shift ) SendFunc ( ERASE1 );	/* shift erase */
	else if ( value == 0x7f ) SendKey ( 0x08 );		/* rubout */
	else if ( value < 0x7f ) SendKey ( value );		/* other */
}

/**************************************************************************
 *
 *	void SendFunc ( value )
 *
 *	Send a function key out to plato ( see pkeyboard.h for list )
 *
 *************************************************************************/

void	SendFunc ( value )

WORD	value;

{
	SendKey ( FuncKeyMap [ value ] );
}

/**************************************************************************
 *
 *	void SendNum ( value, state )
 *
 *	Send a number back ( based on the state of the kbd )
 *
 *************************************************************************/

void	SendNum ( value, state )

WORD	value;
WORD	state;

{
	if ( state == 0x00 )	 		/* straight number */
		SendKey ( 0x30 + value );

	else if ( state == SHIFTED ) {		/* send from the top */
		switch ( value ) {
			case	0x00 :	SendKey ( 0x29 );	/* ')' */
					break;
			case	0x01 :	SendKey ( 0x21 );	/* '!' */
					break;
			case	0x02 :	SendKey ( 0x7b );	/* '@' */
					SendKey ( 0x35 );	
					break;
			case	0x03 :	SendKey ( 0x7b );	/* '#' */
					SendKey ( 0x24 );
					break;
			case	0x04 : 	SendKey ( 0x24 );	/* '$' */
					break;
			case	0x05 :	SendKey ( 0x25 );	/* '%' */
					break;
			case	0x06 :	SendKey ( 0x7b );	/* '^' */
					SendKey ( 0x78 );
					break;
			case	0x07 :	SendKey ( 0x7b );	/* '&' */
					SendKey ( 0x2b );
					break;
			case	0x08 :	SendKey ( 0x2a );	/* '*' */
					break;
			case	0x09 :	SendKey ( 0x28 );	/* '(' */
					break;
		}
	}
	
	else if ( state == ALTED ) 		/* send a plato keyboard */
		SendKey ( AltNumMap [ value ] );

	else if ( state == ( ALTED | SHIFTED ) ) {	/* send a color key */
		SendKey ( 0x00 );			/* send ACCESS */
		SendKey ( 0x5c );			/* send Shift */
		SendKey ( value + 0x30 );		/* send number */
	}
}

/**************************************************************************
 *
 *	void SendKey ( value )
 *
 *	Send a key out to plato ( do flow translation here )
 *
 *************************************************************************/

void	SendKey ( value )

WORD	value;

{
	if ( !flowActive ) SendChar ( value );	/* no flow translation */

	else {					/* translate flow */
		switch ( value ) {
			case	0x00 :	SendChar ( 0x1b );
					SendChar ( 0x1d );
					break;
			case	0x05 :	SendChar ( 0x1b );
					SendChar ( 0x04 );
					break;
			case	0x09 :	SendChar ( 0x0a );
					break;
			case	0x0a :	SendChar ( 0x09 );
					break;
			case	0x11 :	SendChar ( 0x05 );
					break;
			case	0x13 :	SendChar ( 0x17 );
					break;
			case	0x17 :	SendChar ( 0x1b );
					SendChar ( 0x17 );
					break;
			case	0x27 :	SendChar ( 0x7c );
					break;
			case	0x7c :	SendChar ( 0x27 );
					break;
			default :	SendChar ( value );
		}
	}
}
