/****************************************************************************
 *
 * Ethernet IO Support (4.2 custom for Sun)
 *
 * by Kurt Mahan
 *
 ***************************************************************************/

#include	<sys/types.h>
#include	<sys/socket.h>
#include	<sys/un.h>
#include	<netinet/in.h>
#include	<netdb.h>

#include <stdio.h>
#include <sgtty.h>
#include <fcntl.h>
#include <signal.h>
#include <suntool/sunview.h>
#include <suntool/canvas.h>


/**************************************************************************
 *
 *	Global Variables
 *
 *************************************************************************/

struct	sockaddr_un	serveru;
struct	sockaddr_in	serveri;
struct	servent		*sp;
struct	hostent		*hp;

int	sock;
int	inPtr, outPtr;

unsigned char	inChars[0x8000];

extern char	unix_sock_name[],
		inet_serv_name[],
		inet_host_name[];

extern int	use_inet_sock;
extern int	inet_serv_byname;
extern int 	inet_serv_port;
extern int	inet_host_byname;
extern unsigned long	inet_host_addr;

/**************************************************************************
 **************************************************************************
 *
 *	Routines
 *
 **************************************************************************
 *************************************************************************/

/**************************************************************************
 *
 *	EtherInit ( )
 *
 *************************************************************************/

void EtherInit ()
{
	if (use_inet_sock)
	{
		bzero((char *)&serveri, sizeof (serveri));
		serveri.sin_family = AF_INET;

		if (inet_serv_byname) {
			sp = getservbyname ( inet_serv_name,"tcp" );
			if (sp == NULL) {
				fprintf(stderr,"Unknown service '%s'/tcp\n",
					inet_serv_name);
				exit(1);
			}
			serveri.sin_port = sp->s_port;
		} else
			serveri.sin_port = inet_serv_port;

		if (inet_host_byname) {
			hp = gethostbyname(inet_host_name);
			if (hp == NULL) {
				fprintf(stderr,"Unknown host '%s'\n",
					inet_host_name);
				exit(1);
			}
			bcopy(hp->h_addr,(char *)&serveri.sin_addr,
				hp->h_length);
		} else
			bcopy((char *)&inet_host_addr,(char *)&serveri.sin_addr,
				4);

		sock = socket ( AF_INET, SOCK_STREAM, 0 );
		if ( sock < 0 )
		{
			fprintf(stderr,"pad: socket problems\n");
			exit(1);
		}

		if (connect(sock,(struct sockaddr *)&serveri, sizeof(serveri)) < 0)
		{
			fprintf(stderr,"pad: unable to connect\n");
			exit(1);
		}
	}
	else
	{
		sock = socket ( AF_UNIX, SOCK_STREAM, 0 );
		if ( sock < 0 )
		{
			fprintf("pad: socket problems\n");
			exit(1);
		}
	
		serveru.sun_family = AF_UNIX;
		strcpy(serveru.sun_path,unix_sock_name);
	
		if (connect(sock,(struct sockaddr *)&serveru,sizeof(serveru)) < 0)
		{
			fprintf("pad: unable to connect\n");
			exit(1);
		}
	}

	inPtr = outPtr = 0;

	fcntl(sock,F_SETFL,FNDELAY | FASYNC) ;
	fcntl(sock,F_SETOWN,getpid());			/* kinda important */

	EtherInt();
}



/**************************************************************************
 *
 *	EtherClose ( )
 *
 *	Close the currently open ethernet port 
 *
 *************************************************************************/

void EtherClose ( )

{		
	if ( sock> 0 ) close ( sock );
	sock = -1;
}

/**************************************************************************
 *
 *	CharGet ( ) 
 *
 *	Return next character from specified port
 *
 *************************************************************************/

int	CharGet ( )

{
	char	ch;

	if ( inPtr == outPtr ) return -1;
	ch = inChars [ outPtr ];
	outPtr = ( outPtr + 1 ) & 0x7fff;
	return ( ch & 0xff );
}
	
/**************************************************************************
 *
 *	SendChar ( value ) 
 *
 *	Send *value* out the specified serial port
 *
 *************************************************************************/

void SendChar ( value )

char value;

{
	if ( sock > 0 ) 
		write ( sock , &value, 1 );
}

/**************************************************************************
 *
 *	EtherInt ()
 *
 *	4.2 interrupt routine
 *
 *************************************************************************/

EtherInt ()
{
	int	n;
	int	i;
	unsigned char	ch [ 0x100 ];


	n = read ( sock, ch, 0x100 );		/* read a char */
	if ( n == 0 ) {
		fprintf("pad: connection closed by foreign host\n");
		exit(1);
	}
	if ( n > 0 ) {				/* got a char */
		for ( i = 0; i < n; i++ ) {
			inChars [ inPtr ] = ch [ i ] ;
			inPtr = ( inPtr + 1 ) & 0x7fff;
		}
	}
	return	NOTIFY_DONE;
}
