/**************************************************************************
 *
 *	Sun Text Support
 *
 *************************************************************************/

/**************************************************************************
 *
 *	Includes
 *
 *************************************************************************/

#include	<stdio.h>
#include	<sys/file.h>
#include	<suntool/sunview.h>
#include	<suntool/canvas.h>
#include	<pixrect/pixrect_hs.h>
#include	<pixrect/memvar.h>
 
#include 	"machdef.h"
#include 	"types.h"
#include 	"engine.h"
#include	"text.h"

/**************************************************************************
 *
 *	External Global Variables
 *
 *************************************************************************/
 
extern WORD	textCharset;
extern BYTE 	textDir;
extern BYTE 	textAxis;
extern BYTE 	textSize;
extern WORD 	currentX, currentY;
extern WORD 	screenMode;
extern DOUBLE 	memAddr;
extern WORD 	centerX, centerY;

extern Pixwin	*pw;
extern int	setBatch;
extern char	setFont [];
extern int	setBase;

/**************************************************************************
 *
 *	Global Variables
 *
 *************************************************************************/
 
WORD 	numChars [] = { 144, 1, 90, 90 };
WORD 	charDownLoad;
WORD	charBase;

struct pixrect	*charMem [ 4 ];
struct pixrect	*boldChar;

BYTE	boldNibble [ 16 ] = { 	0x00, 0x03, 0x0c, 0x0f,
				0x30, 0x33, 0x3c, 0x3f,
				0xc0, 0xc3, 0xcc, 0xcf,
				0xf0, 0xf3, 0xfc, 0xff };

/**************************************************************************
 *
 *	Routines
 *
 *************************************************************************/
 
/**************************************************************************
 *
 *	InitText () 
 *
 *	Initialize text plotting routines
 *
 *************************************************************************/
 
void InitText ()

{
	int	i;
	FILE	*m0File;
	int	fileDes;

	/*
	 *	Try and load up the m0 charset from disk
 	 */

	fileDes = open ( setFont,O_RDONLY );
	m0File = fdopen ( fileDes, "r" );

	if ( m0File == NULL ) {
		printf ("Could not open m0 charset file : %s\n",setFont);
		i = 0;
	}
	else {
		charMem [ 0 ] = pr_load ( m0File, NULL );
		if ( charMem [ 0 ] == NULL ) {
			printf ("Error in loading m0 font.\n");
			i = 0;
		}
		else {
			i = 1;		/* everything is cool */
			charDownLoad = 3;
		}
	}
	if ( m0File != NULL ) fclose ( m0File );

	for (; i < 4; i++ ) {
		charMem [ i ] = mem_create ( numChars[i]*8,16,1 );
		if ( charMem [ i ] == NULL ) 
			printf ("Error in creating charset memory %d\n",i);
	}

	boldChar = mem_create ( 16, 32, 1);
}

/**************************************************************************
 *
 *	PlotChar ( value ) 
 *
 *	Plot character at given location 
 *
 *************************************************************************/

void PlotChar ( value )

WORD value ;

{
	int	plotMode;
	int	i;
	BYTE	tempByte;
	int	tempCharset = textCharset;

	plotMode = PenMode ();

	currentY = currentY & 511;

	if ( (charDownLoad != 3 ) && ( textCharset < 2 ) )
		pw_char ( pw,currentX, 512-currentY, PIX_SRC, NULL, value );
	else {
		if ( textCharset == 1 ) {
			tempCharset = 0;
			value += 0x60;
		}
		value -= 0x20;

		if ( setBatch ) pw_batch_on ( pw );
		
		if ( !textSize ) 			/* size 0 */
			pw_rop ( pw, currentX, 497-currentY, 8,16, plotMode,
				charMem[tempCharset], value * 8, 0 );
		else {					/* bold */
			for ( i = 0; i < 16; i++ ) {
				tempByte = *(mprd8_addr( mpr_d(charMem[tempCharset]) , value*8, i,1 ) ) ; 
				*(mprd8_addr(mpr_d(boldChar), 0,i<<1,1) ) = boldNibble [ (tempByte >> 4) &0x0f ];
				*(mprd8_addr(mpr_d(boldChar), 0,(i<<1)+1,1)) = boldNibble [ (tempByte >> 4) & 0x0f ];
				*(mprd8_addr(mpr_d(boldChar), 0,i<<1,1)+1) = boldNibble [ tempByte & 0x0f ];
				*(mprd8_addr(mpr_d(boldChar), 0,(i<<1)+1,1)+1) = boldNibble [ tempByte & 0x0f ];
			}
			pw_rop ( pw, currentX, 481-currentY, 16,32, plotMode,
				boldChar, 0, 0 );
		}

	}
		
}

/*************************************************************************
 *
 *	LoadAddrChar ( charArray )
 *
 *	Load the character in charArray [] with scaling into the stuff 
 *	pointed to by *memAddr*
 *
 ************************************************************************/
 
void LoadAddrChar ( charArray )

BYTE	charArray [];

{
	WORD	charNum;		/* charset working on */
	WORD	charSet;
	WORD	i;
	WORD	badChar;		/* if true, don't load */

	charNum = (( memAddr - setBase ) >> 4 ) - 1;
	badChar = False;

	if ( !charDownLoad || ( charDownLoad == 3 ) ) {		/* normal charset download */
		if ( ( charNum < 0 ) || ( charNum > 127 ) ) {	/* bad charset number */
			printf ("Bad charset load number : %d\n", charNum );
			charNum = 0;
			badChar = True;
		}
		if ( charNum > 63 ) {				/* M3 charset */
			charSet = 3;
			charNum -= 64;
		}
		else charSet = 2;				/* M2 charset */
	}
	else {							/* loading M0/M1 charset */
		i = charNum ;
		if ( charDownLoad == 1 ) charNum = xlatChars [ charNum ] ;
		else charNum = exXlatChars [ charNum ] ;
		if ( charNum & 0x80 ) charSet = 1;		/* M1 charset */
		else charSet = 0;				/* M0 charset */

		if (( !charNum ) && ( i ) ) {			/* blow off this char */
			Beep ();
			printf ("Trying to load bad m%d character %d\n",charSet,charNum );
			badChar = True;
		}
		else {
			charNum &= 0x7f;			/* mask off top bit */
			if ( charSet == 1 ) {			/* convert to M0 chars */
				charSet = 0;			/* stuff at end of M0 charset */
				charNum += 0x60;
			}
		}
	}
	
	/*
 	 *	Load the charArray into the character pointed to by *charSet* / *charNum*
	 */

	if ( !badChar ) {
		for ( i = 0; i < 16; i++ ) 
			*(mprd8_addr( mpr_d(charMem[charSet]) , charNum*8, i,1 ) ) = charArray [ i ]; 
	}
		
}

/*************************************************************************
 *
 *	TextKey ()
 *
 *	Called when the R11 key is pressed
 *
 ************************************************************************/
 
int	TextKey ()

{
	FILE	*m0File;
	int	fileDes;

	if ( charDownLoad == 0 ) {
		printf ("Download regular text chars \n");
		charDownLoad = 1;
	}
	else if ( charDownLoad == 1 ) {
		printf ("Download special chars \n");
		charDownLoad = 2;
	}
	else if ( charDownLoad == 2 ) {
		fileDes = open ( setFont, O_WRONLY|O_CREAT );
		m0File = fdopen ( fileDes, "w" );
		if ( m0File == NULL ) printf ("Could not open file for writing..\n");
		else {
			printf ("Saving charset : %x\n",pr_dump ( charMem [ 0 ] , m0File, NULL, RT_STANDARD, 0));
			fclose ( m0File );
		}
		printf ("Using text charset..\n");
		charDownLoad = 3;
	}
	else {
		printf ("Leaving the plato charset..\n");
		charDownLoad = 0;
	}
}
